/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.api;

import java.util.List;

import org.apache.commons.cli.Options;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.so.api.exception.SubscriptionObjectMissingOptionsException;

/**
 * The interface of a subscription object (an object emitting defect
 * notification)
 * 
 * @author Christophe DENEUX - Linagora
 */
public interface SubscriptionObject {

    /**
     * @return the subscription object name. Not <code>null</code>.
     */
    public String getName();

    /**
     * @return the subscription object description. Not <code>null</code>.
     */
    public String getDescription();

    /**
     * @return the list of defects that can be emitted by the subscription
     *         object. Not <code>null</code>.
     */
    public List<EmittableDefect> getEmittableDefects();

    /**
     * Subscribes to the emittable defects of this subscription object.
     * 
     * @param args
     *            Arguments of the subscription object
     * @throws SubscriptionObjectBadArgumentNumberException
     *             Bad argument number for the subscription object
     * @throws SubscriptionObjectMissingArgumentException
     *             An argument is missing for one option of the subscription
     *             object
     * @throws SubscripitonObjectMissingOptionsException
     *             An required option of the subscription object is missing
     * @throws SubscriptionObjectException
     *             An error occurs subscribing to the defect
     */
    public void subscribe(final String[] args) throws SubscriptionObjectBadArgumentNumberException,
            SubscriptionObjectMissingArgumentException, SubscriptionObjectMissingOptionsException,
            SubscriptionObjectException;

    /**
     * Unsubscribes from the emittable defects of this subscription object.
     * 
     * @param args
     *            Arguments of the subscription object
     * @throws SubscriptionObjectBadArgumentNumberException
     *             Bad argument number for the subscription object
     * @throws SubscriptionObjectException
     *             An error occurs subscribing to the defect
     */
    public void unsubscribe(final String[] args)
            throws SubscriptionObjectBadArgumentNumberException, SubscriptionObjectException;

    /**
     * @return The subscription object options
     */
    public Options getOptions();

    /**
     * After each arguments parsing, it is needed to reset {@link Options} of
     * the subscription object.
     */
    public void resetOptions();

    /**
     * Define the Petals CLI shell in which the subscription object is running
     * 
     * @param shell
     *            The Petals CLI shell in which the subscription object is
     *            running
     */
    public void setShell(final Shell shell);

}
